@@ -2,6 +2,7 @@
 import abc
 import unittest
 import datetime
+from collections import defaultdict
 
 
 ################################################################################
@@ -16,7 +17,7 @@ class Environment:
 
     __metaclass__ = abc.ABCMeta
 
-    def process_answer(self, user, item, asked, answered, time, response_time=None, **kwargs):
+    def process_answer(self, user, item, asked, answered, time, response_time, **kwargs):
         """
         This method is used during the answer streaming and is called after the
         predictive model for each answer.
@@ -37,27 +38,43 @@ class Environment:
         pass
 
     @abc.abstractmethod
-    def audit(self, key, user=None, item=None, item_secondary=None, limit=None):
+    def audit(self, key, user=None, item=None, item_secondary=None, limit=None, symmetric=True):
         pass
 
     @abc.abstractmethod
-    def read(self, key, user=None, item=None, item_secondary=None, default=None):
+    def get_items_with_values(self, key, item, user=None):
         pass
 
     @abc.abstractmethod
-    def read_more_items(self, key, items, user=None, item=None, default=None):
+    def get_items_with_values_more_items(self, key, items, user=None):
         pass
 
     @abc.abstractmethod
-    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True):
+    def read(self, key, user=None, item=None, item_secondary=None, default=None, symmetric=True):
         pass
 
-    def update(self, key, init_value, update_fun, user=None, item=None, item_secondary=None, time=None, audit=True):
+    @abc.abstractmethod
+    def read_more_items(self, key, items, user=None, item=None, default=None, symmetric=True):
+        pass
+
+    @abc.abstractmethod
+    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True, symmetric=True):
+        pass
+
+    def update(self, key, init_value, update_fun, user=None, item=None, item_secondary=None, time=None, audit=True, symmetric=True):
         value = self.read(
-            key, user=user, item=item, item_secondary=item_secondary, default=init_value)
+            key, user=user, item=item, item_secondary=item_secondary, default=init_value, symmetric=symmetric)
         self.write(
             key, update_fun(value), user=user,
-            item=item, item_secondary=item_secondary, time=time, audit=audit)
+            item=item, item_secondary=item_secondary, time=time, audit=audit, symmetric=symmetric)
+
+    @abc.abstractmethod
+    def export_values(self):
+        pass
+
+    @abc.abstractmethod
+    def export_audit(self):
+        pass
 
     def flush(self):
         """
@@ -117,10 +134,10 @@ class InMemoryEnvironment(CommonEnvironment):
     CONFUSING_FACTOR = 'confusing_factor'
 
     def __init__(self):
-        self._audit = {}
-        self._state = {}
+        # key -> user -> item_primary -> item_secondary -> [(time, value)]
+        self._data = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list))))
 
-    def process_answer(self, user, item, asked, answered, time, response_time=None, **kwargs):
+    def process_answer(self, user, item, asked, answered, time, response_time, **kwargs):
         if time is None:
             time = datetime.datetime.now()
 
@@ -139,29 +156,45 @@ class InMemoryEnvironment(CommonEnvironment):
             self.update(self.CONFUSING_FACTOR, 0, increment, item=asked, item_secondary=answered)
             self.update(self.CONFUSING_FACTOR, 0, increment, item=asked, item_secondary=answered, user=user)
 
-    def audit(self, key, user=None, item=None, item_secondary=None, limit=None):
-        audit = self._audit.get(self._key(key, user, item, item_secondary), [])
+    def audit(self, key, user=None, item=None, item_secondary=None, limit=None, symmetric=True):
+        items = [item_secondary, item]
+        if symmetric:
+            items = sorted(items)
+        found = list(self._data[key][user][items[1]][items[0]])
         if limit is not None:
-            audit = audit[-limit:]
-        return audit
+            found = found[-limit:]
+        found.reverse()
+        return found
 
-    def read(self, key, user=None, item=None, item_secondary=None, default=None):
-        return self._state.get(self._key(key, user, item, item_secondary), default)
+    def get_items_with_values(self, key, item, user=None):
+        return map(lambda (i, l): (i, l[-1][1]), self._data[key][user][item].items())
 
-    def read_more_items(self, key, items, user=None, item=None, default=None):
+    def get_items_with_values_more_items(self, key, items, user=None):
+        return map(lambda i: self.get_items_with_values(key, i, user), items)
+
+    def read(self, key, user=None, item=None, item_secondary=None, default=None, symmetric=True):
+        found = self.audit(key, user, item, item_secondary, limit=1, symmetric=symmetric)
+        if found:
+            return found[-1][1]
+        else:
+            return default
+
+    def read_more_items(self, key, items, user=None, item=None, default=None, symmetric=True):
         return map(
-            lambda i: self.read(key, user, i, item, default),
+            lambda i: self.read(key, user, i, item, default, symmetric),
             items)
 
-    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True):
-        _key = self._key(key, user, item, item_secondary)
+    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True, symmetric=True):
+        items = [item_secondary, item]
+        if symmetric:
+            items = sorted(items)
         if time is None:
             time = datetime.datetime.now()
-        if audit:
-            previous_audit = self._audit.get(_key, [])
-            previous_audit.append((time, value))
-            self._audit[_key] = previous_audit
-        self._state[_key] = value
+        found = self._data[key][user][items[1]][items[0]]
+        if audit or not found:
+            found.append((time, value))
+        else:
+            found[-1] = (time, value)
 
     def number_of_answers(self, user=None, item=None):
         return self.read(self.NUMBER_OF_ANSWERS, user=user, item=item, default=0)
@@ -195,9 +228,22 @@ class InMemoryEnvironment(CommonEnvironment):
     def confusing_factor_more_items(self, item, items, user=None):
         return self.read_more_items(self.CONFUSING_FACTOR, item=item, items=items, user=user, default=0)
 
-    def _key(self, key, user, item, item_secondary):
-        items = sorted([item, item_secondary])
-        return (key, user, items[1], items[0])
+    def export_values(self):
+        for key, users in self._data.iteritems():
+            for user, primaries in users.iteritems():
+                for item_primary, secondaries in primaries.iteritems():
+                    for item_secondary, values in secondaries.iteritems():
+                        if len(values) > 0:
+                            time, value = values[-1]
+                            yield (key, user, item_primary, item_secondary, time, value)
+
+    def export_audit(self):
+        for key, users in self._data.iteritems():
+            for user, primaries in users.iteritems():
+                for item_primary, secondaries in primaries.iteritems():
+                    for item_secondary, values in secondaries.iteritems():
+                        for time, value in values:
+                            yield (key, user, item_primary, item_secondary, time, value)
 
 
 ################################################################################
@@ -258,8 +304,11 @@ class TestEnvironment(unittest.TestCase):
         for value in range(100):
             env.write('key', value)
         expected = map(float, range(100))
+        expected.reverse()
         found = list(zip(*env.audit('key'))[1])
         self.assertEqual(expected, found)
+        found = list(zip(*env.audit('key', limit=10))[1])
+        self.assertEqual(expected[:10], found)
 
 
 class TestCommonEnvironment(TestEnvironment):
@@ -305,6 +354,18 @@ class TestCommonEnvironment(TestEnvironment):
         self.assertEqual(2, env.number_of_first_answers(item=items[0]))
         self.assertEqual([2 for i in items], env.number_of_first_answers_more_items(items))
 
+    def test_symmetry(self):
+        env = self.generate_environment()
+        items = [self.generate_item() for i in range(2)]
+        env.write(key='test_symmetric', value=1, item=items[0], item_secondary=items[1])
+        env.write(
+            key='test_assymetric', value=1, item=items[0],
+            item_secondary=items[1], symmetric=False)
+        self.assertEqual(
+            1, env.read(key='test_symmetric', item=items[1], item_secondary=items[0]))
+        self.assertEqual(
+            None, env.read(key='test_assymetric', item=items[1], item_secondary=items[0]))
+
     def test_last_answer_time(self):
         env = self.generate_environment()
         user_1 = self.generate_user()
@@ -318,7 +379,7 @@ class TestCommonEnvironment(TestEnvironment):
         for u in [user_1, user_2]:
             for i in items:
                 for j in range(10):
-                    env.process_answer(u, i, i, i, datetime.datetime.now())
+                    env.process_answer(u, i, i, i, datetime.datetime.now(), 1000)
         self.assertIsNotNone(env.number_of_first_answers())
         self.assertIsNotNone(env.number_of_first_answers(user=user_1))
         self.assertIsNotNone(env.number_of_first_answers(user=user_1, item=items[0]))
@@ -336,7 +397,7 @@ class TestCommonEnvironment(TestEnvironment):
         for u in [user_1, user_2]:
             for i in items:
                 for j in range(10):
-                    env.process_answer(u, i, i, i + diff, datetime.datetime.now())
+                    env.process_answer(u, i, i, i + diff, datetime.datetime.now(), 1000)
             diff += 1
         self.assertEqual(1.0, env.rolling_success(user_1))
         self.assertEqual(0.0, env.rolling_success(user_2))
@@ -349,10 +410,35 @@ class TestCommonEnvironment(TestEnvironment):
         self.assertEqual(0, env.confusing_factor(item=items[0], item_secondary=items[1]))
         self.assertEqual(0, env.confusing_factor(item=items[0], item_secondary=items[1], user=user_1))
         for i in items:
-            env.process_answer(user_1, items[0], items[0], i, datetime.datetime.now())
+            env.process_answer(user_1, items[0], items[0], i, datetime.datetime.now(), 1000)
         for i in items:
-            env.process_answer(user_2, i, i, i, datetime.datetime.now())
+            env.process_answer(user_2, i, i, i, datetime.datetime.now(), 1000)
         self.assertEqual(1, env.confusing_factor(item=items[0], item_secondary=items[1]))
         self.assertEqual(1, env.confusing_factor(item=items[0], item_secondary=items[1], user=user_1))
         self.assertEqual(0, env.confusing_factor(item=items[0], item_secondary=items[1], user=user_2))
         self.assertEqual(0, env.confusing_factor(item=items[2], item_secondary=items[3]))
+
+    def test_get_items_with_values(self):
+        env = self.generate_environment()
+        users = [self.generate_user() for i in range(2)]
+        items = [self.generate_item() for i in range(2)]
+        env.write('parent', 10, user=users[0], item=items[0], item_secondary=items[1], symmetric=False)
+        env.write('parent', 20, item=items[1], item_secondary=items[0], symmetric=False)
+        self.assertEqual([(items[1], 10)], env.get_items_with_values('parent', user=users[0], item=items[0]))
+        self.assertEqual([], env.get_items_with_values('parent', user=users[0], item=items[1]))
+        self.assertEqual([], env.get_items_with_values('parent', user=users[1], item=items[0]))
+        self.assertEqual([(items[0], 20)], env.get_items_with_values('parent', item=items[1]))
+        self.assertEqual([], env.get_items_with_values('parent', user=users[0], item=items[1]))
+
+    def test_get_items_with_values_more_items(self):
+        env = self.generate_environment()
+        users = [self.generate_user() for i in range(2)]
+        items = [self.generate_item() for i in range(2)]
+        env.write('parent', 10, user=users[0], item=items[0], item_secondary=items[1], symmetric=False)
+        env.write('parent', 20, item=items[1], item_secondary=items[0], symmetric=False)
+        self.assertEqual(
+            [[(items[1], 10)], []],
+            env.get_items_with_values_more_items('parent', user=users[0], items=items))
+        self.assertEqual(
+            [[], [(items[0], 20)]],
+            env.get_items_with_values_more_items('parent', items=items))
