@@ -1,6 +1,6 @@
 from django.contrib.auth.models import User
 from django.db import models
-from django.db.models.signals import pre_save, post_save
+from django.db.models.signals import post_save
 from django.dispatch import receiver
 from proso.models.environment import CommonEnvironment, InMemoryEnvironment
 from datetime import datetime
@@ -10,6 +10,12 @@ from django.conf import settings
 import re
 import os.path
 import proso.util
+from decorator import cache_environment_for_item
+from collections import defaultdict
+
+
+# This is hack to emulate TRUE value on both psql and sqlite
+DATABASE_TRUE = '1 = 1'
 
 
 ################################################################################
@@ -25,7 +31,7 @@ def get_predictive_model():
 
 
 def get_recommendation():
-    return proso.util.instantiate(settings.PROSO_RECOMMENDATION, [get_predictive_model()])
+    return proso.util.instantiate(settings.PROSO_RECOMMENDATION, get_predictive_model())
 
 
 ################################################################################
@@ -42,30 +48,31 @@ class InMemoryDatabaseFlushEnvironment(InMemoryEnvironment):
         filename_audit = os.path.join(settings.DATA_DIR, 'environment_flush_audit.csv')
         filename_variable = os.path.join(settings.DATA_DIR, 'environment_flush_variable.csv')
         with open(filename_audit, 'w') as file_audit:
-            for (key, u, i_p, i_s), values in self._audit.iteritems():
+            for (key, u, i_p, i_s, t, v) in self.export_audit():
                 if key in to_skip:
                     continue
-                for (t, v) in values:
-                    file_audit.write(
-                        ('%s,%s,%s,%s,%s,%s\n' % (key, u, i_p, i_s, t.strftime('%Y-%m-%d %H:%M:%S'), v)).replace('None', ''))
+                file_audit.write(
+                    ('%s,%s,%s,%s,%s,%s\n' % (key, u, i_p, i_s, t.strftime('%Y-%m-%d %H:%M:%S'), v)).replace('None', ''))
         with open(filename_variable, 'w') as file_variable:
-            current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
-            for (key, u, i_p, i_s), v in self._state.iteritems():
+            for (key, u, i_p, i_s, t, v) in self.export_values():
                 if key in to_skip:
                     continue
                 file_variable.write(
-                    ('%s,%s,%s,%s,%s,%s,%s\n' % (key, u, i_p, i_s, v, 0, current_time)).replace('None', ''))
-        print 'DELETE FROM proso_models_audit;'
-        print 'DELETE FROM proso_models_variable;'
+                    ('%s,%s,%s,%s,%s,%s,%s\n' % (key, u, i_p, i_s, v, 0, t)).replace('None', ''))
+        print 'BEGIN;'
+        print 'SET CONSTRAINTS ALL DEFERRED;'
+        print "DELETE FROM proso_models_audit;"
+        print "DELETE FROM proso_models_variable;"
         print "\copy proso_models_audit (key, user_id, item_primary_id, item_secondary_id, time, value) FROM '%s' WITH (FORMAT csv);" % filename_audit
         print "\copy proso_models_variable (key, user_id, item_primary_id, item_secondary_id, value, audit, updated) FROM '%s' WITH (FORMAT csv);" % filename_variable
+        print 'COMMIT;'
 
 
 class DatabaseEnvironment(CommonEnvironment):
 
     time = None
 
-    def process_answer(self, user, item, asked, answered, time, response_time=None, **kwargs):
+    def process_answer(self, user, item, asked, answered, time, response_time, **kwargs):
         answer = Answer(
             user_id=user,
             item_id=item,
@@ -75,9 +82,9 @@ class DatabaseEnvironment(CommonEnvironment):
             response_time=response_time)
         answer.save()
 
-    def audit(self, key, user=None, item=None, item_secondary=None, limit=100000):
+    def audit(self, key, user=None, item=None, item_secondary=None, limit=100000, symmetric=True):
         with closing(connection.cursor()) as cursor:
-            where, where_params = self._where_single(key, user, item, item_secondary)
+            where, where_params = self._where_single(key, user, item, item_secondary, symmetric)
             cursor.execute(
                 'SELECT time, value FROM proso_models_audit WHERE '
                 + where +
@@ -87,9 +94,44 @@ class DatabaseEnvironment(CommonEnvironment):
             map(lambda (d, v): (self._ensure_is_datetime(d), v), result)
             return result
 
-    def read(self, key, user=None, item=None, item_secondary=None, default=None):
+    def get_items_with_values(self, key, item, user=None):
         with closing(connection.cursor()) as cursor:
-            where, where_params = self._where_single(key, user, item, item_secondary)
+            where, where_params = self._where_single(
+                key, user, item, None, force_null=False, symmetric=False, time_shift=False)
+            cursor.execute(
+                '''
+                SELECT
+                    item_secondary_id,
+                    value
+                FROM
+                    proso_models_variable
+                WHERE
+                ''' + where, where_params)
+            return cursor.fetchall()
+
+    @cache_environment_for_item()
+    def get_items_with_values_more_items(self, key, items, user=None):
+        with closing(connection.cursor()) as cursor:
+            where, where_params = self._where_more_items(
+                key, items, user, None, force_null=['user_id'], symmetric=False, time_shift=False)
+            cursor.execute(
+                '''
+                SELECT
+                    item_primary_id,
+                    item_secondary_id,
+                    value
+                FROM
+                    proso_models_variable
+                WHERE
+                ''' + where, where_params)
+            result = defaultdict(list)
+            for p_id, s_id, val in cursor:
+                result[p_id].append((s_id, val))
+            return map(lambda i: result[i], items)
+
+    def read(self, key, user=None, item=None, item_secondary=None, default=None, symmetric=True):
+        with closing(connection.cursor()) as cursor:
+            where, where_params = self._where_single(key, user, item, item_secondary, symmetric=symmetric)
             if self.time is None:
                 cursor.execute(
                     'SELECT value FROM proso_models_variable WHERE ' + where,
@@ -103,9 +145,10 @@ class DatabaseEnvironment(CommonEnvironment):
                 else:
                     return audit[0][1]
 
-    def read_more_items(self, key, items, user=None, item=None, default=None):
+    @cache_environment_for_item()
+    def read_more_items(self, key, items, user=None, item=None, default=None, symmetric=True):
         with closing(connection.cursor()) as cursor:
-            where, where_params = self._where_more_items(key, items, user, item)
+            where, where_params = self._where_more_items(key, items, user, item, symmetric=symmetric)
             if self.time is None:
                 cursor.execute(
                     'SELECT item_primary_id, item_secondary_id, value FROM proso_models_variable WHERE '
@@ -115,12 +158,12 @@ class DatabaseEnvironment(CommonEnvironment):
             else:
                 cursor.execute(
                     '''SELECT DISTINCT ON
-                        (key, item_primary_id, item_secondary_id, user_id, item_primary_id)
-                        item_secondary_id, value FROM proso_models_audit WHERE
+                        (key, item_primary_id, item_secondary_id, user_id)
+                        item_primary_id, item_secondary_id, value FROM proso_models_audit WHERE
                     ''' + where +
-                    ' ORDER BY time',
+                    ' ORDER BY key, item_primary_id, item_secondary_id, user_id, time',
                     where_params)
-                result = map(lambda (w, x, y, z): (x, y, z), cursor.fetchall())
+                result = cursor.fetchall()
             if item is None:
                 result = map(lambda (x, y, z): (x, z), result)
             else:
@@ -128,12 +171,14 @@ class DatabaseEnvironment(CommonEnvironment):
             result = dict(result)
             return map(lambda key: result.get(key, default), items)
 
-    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True):
+    def write(self, key, value, user=None, item=None, item_secondary=None, time=None, audit=True, symmetric=True):
         if key is None:
             raise Exception('Key has to be specified')
         if value is None:
             raise Exception('Value has to be specified')
-        items = sorted([item, item_secondary])
+        items = [item_secondary, item]
+        if symmetric:
+            items = sorted(items)
         data = {
             'user_id': user,
             'item_primary_id': items[1],
@@ -144,10 +189,11 @@ class DatabaseEnvironment(CommonEnvironment):
             variable = Variable.objects.get(**data)
         except Variable.DoesNotExist:
             variable = Variable(**data)
-        variable.value = value
-        variable.audit = audit
-        variable.updated = datetime.now() if time is None else time
-        variable.save()
+        if variable.value != value:
+            variable.value = value
+            variable.audit = audit
+            variable.updated = datetime.now() if time is None else time
+            variable.save()
 
     def number_of_answers(self, user=None, item=None):
         with closing(connection.cursor()) as cursor:
@@ -173,6 +219,7 @@ class DatabaseEnvironment(CommonEnvironment):
                 + where, where_params)
             return self._ensure_is_datetime(cursor.fetchone()[0])
 
+    @cache_environment_for_item(default=0)
     def number_of_answers_more_items(self, items, user=None):
         with closing(connection.cursor()) as cursor:
             where, where_params = self._where({'user_id': user, 'item_id': items}, False)
@@ -183,6 +230,7 @@ class DatabaseEnvironment(CommonEnvironment):
             fetched = dict(cursor.fetchall())
             return map(lambda i: fetched.get(i, 0), items)
 
+    @cache_environment_for_item(default=0)
     def number_of_first_answers_more_items(self, items, user=None):
         with closing(connection.cursor()) as cursor:
             where, where_params = self._where({'user_id': user, 'item_id': items}, False)
@@ -194,6 +242,7 @@ class DatabaseEnvironment(CommonEnvironment):
             return map(lambda i: fetched.get(i, 0), items)
         return 0
 
+    @cache_environment_for_item()
     def last_answer_time_more_items(self, items, user=None):
         with closing(connection.cursor()) as cursor:
             where, where_params = self._where({'user_id': user, 'item_id': items}, False)
@@ -218,29 +267,67 @@ class DatabaseEnvironment(CommonEnvironment):
                 ORDER BY id DESC
                 LIMIT %s
                 ''', [user, window_size])
-            fetched = map(lambda x: x[0], cursor.fetchall())
+            fetched = map(lambda x: True if x[0] else False, cursor.fetchall())
             if len(fetched) == 0:
                 return 1.0
             else:
                 return sum(fetched) / float(len(fetched))
 
     def confusing_factor(self, item, item_secondary, user=None):
-        pass
+        with closing(connection.cursor()) as cursor:
+            where, where_params = self._where({
+                'item_asked_id': item,
+                'item_answered_id': item_secondary,
+                'user_id': user
+            }, force_null=False)
+            cursor.execute(
+                '''
+                SELECT
+                    COUNT(proso_models_answer.id) AS confusing_factor
+                FROM
+                    proso_models_answer
+                WHERE
+                ''' + where, where_params)
+            return cursor.fetchone()[0]
 
     def confusing_factor_more_items(self, item, items, user=None):
+        with closing(connection.cursor()) as cursor:
+            where, where_params = self._where({
+                'item_asked_id': item,
+                'item_answered_id': items,
+                'user_id': user
+            }, force_null=False)
+            cursor.execute(
+                '''
+                SELECT
+                    item_answered_id,
+                    COUNT(id) AS confusing_factor
+                FROM
+                    proso_models_answer
+                WHERE
+                ''' + where + ' GROUP BY item_answered_id', where_params)
+            wrongs = dict(cursor.fetchall())
+            return map(lambda i: wrongs.get(i, 0), items)
+
+    def export_values():
         pass
 
-    def _where_single(self, key, user=None, item=None, item_secondary=None):
+    def export_audit():
+        pass
+
+    def _where_single(self, key, user=None, item=None, item_secondary=None, force_null=True, symmetric=True, time_shift=True):
         if key is None:
             raise Exception('Key has to be specified')
-        items = sorted([item, item_secondary])
+        items = [item_secondary, item]
+        if symmetric:
+            items = sorted(items)
         return self._where({
             'user_id': user,
             'item_primary_id': items[1],
             'item_secondary_id': items[0],
-            'key': key})
+            'key': key}, force_null=force_null, time_shift=time_shift)
 
-    def _where_more_items(self, key, items, user=None, item=None, default=None):
+    def _where_more_items(self, key, items, user=None, item=None, force_null=True, symmetric=True, time_shift=True):
         if key is None:
             raise Exception('Key has to be specified')
         cond_secondary = {
@@ -249,8 +336,8 @@ class DatabaseEnvironment(CommonEnvironment):
             'item_primary_id': items,
             'item_secondary_id': item
         }
-        if item is None or all(map(lambda x: item <= x, items)):
-            return self._where(cond_secondary)
+        if item is None or all(map(lambda x: item <= x, items)) or not symmetric:
+            return self._where(cond_secondary, force_null=force_null, time_shift=time_shift)
         cond_primary = {
             'key': key,
             'user_id': user,
@@ -258,17 +345,19 @@ class DatabaseEnvironment(CommonEnvironment):
             'item_secondary_id': items
         }
         if all(map(lambda x: item >= x, items)):
-            return self._where(cond_primary)
+            return self._where(cond_primary, force_null=force_null, time_shift=time_shift)
         return self._where({
             'item is primary': cond_primary,
             'item is secondary': cond_secondary
-        })
+        }, force_null=force_null, time_shift=time_shift)
 
-    def _where(self, condition, force_null=True, top_most=True):
+    def _where(self, condition, force_null=True, top_most=True, time_shift=True):
         if isinstance(condition, tuple):
-            result_cond, result_params = self._column_comparison(condition[0], condition[1], force_null=force_null)
+            result_cond, result_params = self._column_comparison(
+                condition[0], condition[1], force_null=force_null)
         elif isinstance(condition, dict):
-            conds, params = zip(*map(lambda x: self._where(x, force_null, top_most=False), condition.items()))
+            conds, params = zip(*map(
+                lambda x: self._where(x, force_null, top_most=False), condition.items()))
             params = [p for ps in params for p in ps]
             operator = ' AND '
             if any(map(lambda x: isinstance(x, dict), condition)):
@@ -276,29 +365,31 @@ class DatabaseEnvironment(CommonEnvironment):
             result_cond, result_params = operator.join(conds), params
         else:
             raise Exception("Unsupported type of condition:" + str(type(condition)))
-        if top_most and self.time is not None:
+        if top_most and self.time is not None and time_shift:
             result_cond = ('(%s) AND time < ?' % result_cond).replace('?', '%s')
             result_params = result_params + [self.time.strftime('%Y-%m-%d %H:%M:%S')]
         return result_cond, result_params
 
     def _column_comparison(self, column, value, force_null=True):
         if isinstance(value, list):
-            value = filter(lambda x: x is not None, value)
+            value = list(set(value))
+            contains_null = any(map(lambda x: x is None, value))
+            if contains_null:
+                value = filter(lambda x: x is not None, value)
+            null_contains_return = (column + ' IS NULL OR ') if contains_null else ''
             if len(value) > 0:
-                return column + ' IN (' + ','.join(['%s' for i in value]) + ')', value
-            elif force_null:
-                return column + ' IS NULL', []
+                return '(' + null_contains_return + column + ' IN (' + ','.join(['%s' for i in value]) + '))', sorted(value)
             else:
-                return 'TRUE', []
+                return '(' + null_contains_return + DATABASE_TRUE + ')', []
         elif value is not None:
             return column + ' = %s', [value]
-        elif force_null:
+        elif (isinstance(force_null, bool) and force_null) or (isinstance(force_null, list) and column in force_null):
             return column + ' IS NULL', []
         else:
-            return 'TRUE', []
+            return DATABASE_TRUE, []
 
     def _ensure_is_datetime(self, value):
-        if isinstance(value, datetime):
+        if isinstance(value, datetime) or value is None:
             return value
         else:
             matched = re.match(r'([\d -\:]*)\.\d+', value)
@@ -330,7 +421,7 @@ class Answer(models.Model):
         default=None,
         related_name='item_answered_answers')
     time = models.DateTimeField(default=datetime.now)
-    response_time = models.IntegerField(default=0)
+    response_time = models.IntegerField(null=False, blank=False)
 
     class Meta:
         app_label = 'proso_models'
@@ -353,7 +444,7 @@ class Variable(models.Model):
         related_name='item_secondary_variables')
     key = models.CharField(max_length=50)
     value = models.FloatField()
-    audit = models.BooleanField()
+    audit = models.BooleanField(default=True)
     updated = models.DateTimeField(default=datetime.now)
 
     def __str__(self):
@@ -409,25 +500,13 @@ class Audit(models.Model):
 # Signals
 ################################################################################
 
-@receiver(pre_save, sender=Variable)
-def sort_items(sender, instance, **kwargs):
-    if instance.item_primary is None:
-        instance.item_primary = instance.item_secondary
-        instance.item_secondary = None
-    elif instance.item_secondary is not None:
-        if instance.item_primary.pk < instance.item_secondary.pk:
-            backup = instance.item_primary
-            instance.item_primary = instance.item_secondary
-            instance.item_secondary = backup
-
-
 @receiver(post_save, sender=Answer)
 def update_predictive_model(sender, instance, **kwargs):
     environment = get_environment()
     predictive_model = get_predictive_model()
     predictive_model.predict_and_update(
         environment,
-        instance.user.id,
+        instance.user_id,
         instance.item_id,
         instance.item_asked_id == instance.item_answered_id,
         instance.time,
@@ -439,9 +518,12 @@ def update_predictive_model(sender, instance, **kwargs):
 def log_audit(sender, instance, **kwargs):
     if instance.audit:
         audit = Audit(
-            user=instance.user,
+            user_id=instance.user_id,
             item_primary=instance.item_primary,
             key=instance.key,
             value=instance.value,
             time=instance.updated)
         audit.save()
+
+
+PROSO_MODELS_TO_EXPORT = [Answer, Audit, Item, Variable]
